import { Collection, Entry } from "./entry.ts";
import { Store } from "./store.ts";

export type Table<Self extends Table<Self>> = {
	[Name: string]: Structure<Self>;
};

export type Structure<T extends Table<T>> = {
	[name: string]: Property<T>;
};

export type Property<T extends Table<T>> =
	| Field
	| Relation<T>;

export type Field = "string" | "number" | "boolean";
export type Relation<T extends Table<T>> =
	| ["maybe", keyof T]
	| ["one", keyof T]
	| ["many", keyof T];

export type QueriedFor<
	T extends Table<T>,
	P extends Property<T>,
> = P extends "string" ? string
	: P extends "number" ? number
	: P extends "boolean" ? boolean
	: P[0] extends "maybe" ? null | Entry<T, P[1]>
	: P[0] extends "one" ? Entry<T, P[1]>
	: Collection<T, P[1]>;

export type StoredFor<
	T extends Table<T>,
	P extends Property<T>,
> = P extends "string" ? string
	: P extends "number" ? number
	: P extends "boolean" ? boolean
	: P[0] extends "maybe" ? null | string
	: P[0] extends "one" ? string
	: string;

export type QueriedOf<
	T extends Table<T>,
	S extends keyof T,
> = { [P in keyof T[S]]: QueriedFor<T, T[S][P]> };

export type QueriedOfField<
	T extends Table<T>,
	S extends keyof T,
	F extends keyof T[S],
> = QueriedOf<T, S>[F];

export type StoredOfField<
	T extends Table<T>,
	S extends keyof T,
	F extends keyof T[S],
> = { [P in keyof T[S]]: StoredFor<T, T[S][P]> }[F];

export type EntryFor<S extends Store<any>, T extends keyof S["nodes"]["table"]> = Entry<S["nodes"]["table"], T>;
